package communicationmethods;

import library.URLTERMS;
import management.ClientHandling;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

/**
 * Rest Controller, receives Client-requests and responds
 */
@Controller
public class ServerMethods implements IServerMethods {

	private static final String template = "DrawingServer ";

	@RequestMapping(value = URLTERMS.TEST,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
			)
	public @ResponseBody
	ResponseEntity<String> test() {
				return ResponseEntity.ok(new String(" <!DOCTYPE html>\n" +
						"<html>\n" +
						"<head>\n" +
						"<title>Page Title</title>\n" +
						"</head>\n" +
						"<body>\n" +
						"\n" +
						"<h1>My First Heading</h1>\n" +
						"<svg width=\"500\" height=\"500\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">\n" +
						"\n" +
						"\n" +
						"  <circle cx=\"305\" cy=\"200\" r=\"60\" stroke=\"green\" stroke-width=\"4\" fill=\"none\" /> \n" +
						"\n" +
						"<rect x=\"00\" y=\"00\" rx=\"20\" ry=\"20\" width=\"150\" height=\"150\"\n" +
						"  fill=\"blue\" stroke=\"pink\" stroke-width=\"20\" />\n" +
						"<ellipse cx=\"120\" cy=\"350\" rx=\"100\" ry=\"50\" fill=\"green\"\n" +
						"/>\n" +
						"</svg>" +
						"\n" +
						"</body>\n" +
						"</html> "));
	}

	@RequestMapping(value = URLTERMS.CREATESHAPE,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public String createShape(@RequestParam(value="clientid") String clientId,
							  @RequestParam(value="type",defaultValue = "") String type,
							  @RequestParam(value="canvasId",defaultValue = "") String canvasId,
							  @RequestParam(value="cx",defaultValue = "") String cx,
							  @RequestParam(value="cy",defaultValue = "") String cy,
							  @RequestParam(value="rx",defaultValue = "", required = false) String rx,
							  @RequestParam(value="ry",defaultValue = "", required = false) String ry,
							  @RequestParam(value="fill", defaultValue = "BLUE") String fill,
							  @RequestParam(value="stroke",defaultValue = "", required = false) String stroke,
							  @RequestParam(value="stroke_width",defaultValue = "", required = false) String stroke_width,
							  @RequestParam(value="info_text",defaultValue = "", required = false) String info_text) {
		ClientHandling client=new ClientHandling("");
		return client.createNewShape(clientId, type, canvasId, cx, cy, rx, ry,fill, stroke, stroke_width, info_text);
	}

	@RequestMapping(value = URLTERMS.LAYERCREATION,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.POST
	)
	public String createLayer(String clientId, String layerid) {
		ClientHandling client=new ClientHandling(clientId);
		return client.createNewLayer(layerid);

	}

	@RequestMapping(value = URLTERMS.CLIENTREGISTRATION,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.POST
	)
	public String registerClient() {
		ClientHandling client=new ClientHandling("");
		return client.registerNewClient();
	}

	@RequestMapping(value = URLTERMS.SVGDOWNLOAD,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public String getSVG(String clientId) {
		ClientHandling client=new ClientHandling(clientId);
		return client.getSVGDataString();
	}


	@RequestMapping(value = URLTERMS.SHAPEEDIT,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public String editShape(@RequestParam(value="clientid",defaultValue = "") String clientId,
							@RequestParam(value="canvasId",defaultValue = "") String canvasId,
							@RequestParam(value="shapeID",defaultValue = "") String shapeID,
							@RequestParam(value="cx", required = false) String cx,
							@RequestParam(value="cy", required = false) String cy,
							@RequestParam(value="rx", required = false) String rx,
							@RequestParam(value="ry", required = false) String ry,
							@RequestParam(value="fill", required = false) String fill,
							@RequestParam(value="stroke", required = false) String stroke,
							@RequestParam(value="stroke_width", required = false) String stroke_width,
							@RequestParam(value="info_text", required = false) String info_text) {
		ClientHandling client=new ClientHandling(clientId);
		return client.editShape(canvasId, shapeID, cx, cy, rx, ry, fill, stroke, stroke_width, info_text);
	}
	@RequestMapping(value = URLTERMS.GETLAYERS,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public List<String> getLayers(String clientId) {
		ClientHandling client=new ClientHandling(clientId);
		return client.getAllLayerForClient();
	}

	@RequestMapping(value = URLTERMS.GETLAYERSHAPES,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public List<String> getShapesOfLayer(String clientId, String layerId) {
		ClientHandling client=new ClientHandling(clientId);
		return client.getAllShapesFromLayer(layerId);
	}

	@RequestMapping(value = URLTERMS.GETCOLOURS,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public List<String> getColours() {
		ClientHandling client=new ClientHandling("");
		return client.getAllColoursFromLibrary();
	}

	@RequestMapping(value = URLTERMS.GETSHAPES,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public List<String> getShapes() {
		ClientHandling client=new ClientHandling("");
		return client.getAllShapesFromLibrary();
	}

	@RequestMapping(value = URLTERMS.GETEDITOPTIONS,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.GET
	)
	public List<String> getEditOptions() {
		ClientHandling client=new ClientHandling("");
		return client.getAllEditOptionsFromLibrary();
	}


	@RequestMapping(value = URLTERMS.DELETESHAPE,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.DELETE
	)
	public String deleteShape(String clientId, String shapeId)  {
		ClientHandling client=new ClientHandling(clientId);
		if(client.deleteShape(shapeId))return "ok";
		return "Error, could not delete Shape";
	}

	@RequestMapping(value = URLTERMS.DELETECLIENT,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.DELETE
	)
	public String deleteClientData(String clientId)  {
		ClientHandling client=new ClientHandling("");
		return client.deleteAllClienteData();
	}

	@RequestMapping(value = URLTERMS.LAYERACTIVITION,
			//produces = MediaType.TEXT_HTML_VALUE,
			method = RequestMethod.POST
	)
	public String LayerActivition(String clientId, Boolean activated, String layerId) {
		ClientHandling client=new ClientHandling("");
		return client.manageLayerActivity(activated,layerId);
	}

	@Override
	public String getHtmlCanvas(String clientId) {
		ClientHandling client=new ClientHandling(clientId);
		return client.getCanvasHtml();

	}


}
