package operations.create;

import exceptions.serverexceptions.ShapeCreationException;
import library.Colours;
import library.Shape_Forms;
import library.TERMS;
import models.shapes.IShapes;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.logging.Logger;

public class ShapeCreation {
	private Logger log=Logger.getLogger(ShapeCreation.class.getName());
	private ICreateShape shapeCreation;
	private int cx, cy, rx,ry,stroke_width;
	private Colours fill,stroke;
	private String text;

	/**
	 * manages the creation of all shapes
	 */
	public ShapeCreation() {
	}

	/**
	 * discerns which shape to make and what parameters needed
	 * @param shapeType String
	 * @param mainParams Stringlist
	 * @return IShapes
	 * @throws ShapeCreationException if problem with shapeCreation
	 */
	public IShapes createShape(String shapeType, List<String> mainParams) throws ShapeCreationException {
		if (Shape_Forms.CIRCLE.toString().equals(shapeType.toUpperCase())) return createCircle(mainParams);
		if (Shape_Forms.ECLIPSE.toString().equals(shapeType.toUpperCase())) return createEclipse(mainParams);
		if (Shape_Forms.STROKE.toString().equals(shapeType.toUpperCase())) return createStroke(mainParams);
		if (Shape_Forms.RECTANGLE.toString().equals(shapeType.toUpperCase())) return createRectangle(mainParams);
		return null;
	}

	private IShapes createCircle(List<String> params) throws ShapeCreationException {
		try {
			setCx(Integer.parseInt(params.get(TERMS.CX_INDEX)));
			setCy(Integer.parseInt(params.get(TERMS.CY_INDEX)));
			setRx(Integer.parseInt(params.get(TERMS.RX_INDEX)));
		} catch (NumberFormatException e){
			log.warning(e.getMessage());
			throw new ShapeCreationException("error in createCircle() _"+e.getMessage());
		}
		setFill(params.get(TERMS.FILL_INDEX));
		List<Integer> intparams=new ArrayList<>();
		intparams.add(0,getCx());
		intparams.add(1,getCy());
		intparams.add(2,getRx());
		CreateCircle circle = new CreateCircle();
		return circle.create(intparams,getFill(),getID(params.get(TERMS.CANVASID_INDEX)));
	}

	private IShapes createRectangle(List<String> params) throws ShapeCreationException {
		try {
			setCx(Integer.parseInt(params.get(TERMS.CX_INDEX)));
			setCy(Integer.parseInt(params.get(TERMS.CY_INDEX)));
			setRx(Integer.parseInt(params.get(TERMS.RX_INDEX)));
			setRy(Integer.parseInt(params.get(TERMS.RY_INDEX)));
		} catch (NumberFormatException e){
			log.warning(e.getMessage());
			throw new ShapeCreationException("error in createRectangle() _"+e.getMessage());
		}
		setFill(params.get(TERMS.FILL_INDEX));
		List<Integer> intparams=new ArrayList<>();
		intparams.add(0,getCx());
		intparams.add(1,getCy());
		intparams.add(2,getRx());
		intparams.add(3,getRy());
		CreateRectangle rectangle=new CreateRectangle();
		return rectangle.create(intparams,getFill(),getID(params.get(TERMS.CANVASID_INDEX)));
	}

	private IShapes createStroke(List<String> params) throws ShapeCreationException {
		try {
			setCx(Integer.parseInt(params.get(TERMS.CX_INDEX)));
			setCy(Integer.parseInt(params.get(TERMS.CY_INDEX)));
			setRx(Integer.parseInt(params.get(TERMS.RX_INDEX)));
			setRy(Integer.parseInt(params.get(TERMS.RY_INDEX)));
			setStroke_width(Integer.parseInt(params.get(TERMS.STROKEWIDTH_INDEX)));
		} catch (NumberFormatException e){
			log.warning(e.getMessage());
			throw new ShapeCreationException("error in createStroke() _"+e.getMessage());
		}
		setFill(params.get(TERMS.FILL_INDEX));
		List<Integer> intparams=new ArrayList<>();
		intparams.add(0,getCx());
		intparams.add(1,getCy());
		intparams.add(2,getRx());
		intparams.add(3,getRy());
		intparams.add(4,getStroke_width());
		CreateStroke stroke=new CreateStroke();
		return stroke.create(intparams,getFill(),getID(params.get(TERMS.CANVASID_INDEX)));
	}

	private IShapes createEclipse(List<String> params) throws ShapeCreationException {
		try {
			setCx(Integer.parseInt(params.get(TERMS.CX_INDEX)));
			setCy(Integer.parseInt(params.get(TERMS.CY_INDEX)));
			setRx(Integer.parseInt(params.get(TERMS.RX_INDEX)));
			setRy(Integer.parseInt(params.get(TERMS.RY_INDEX)));
		} catch (NumberFormatException e){
			log.warning(e.getMessage());
			throw new ShapeCreationException("error in createRectangle() _"+e.getMessage());
		}
		setFill(params.get(TERMS.FILL_INDEX));
		List<Integer> intparams=new ArrayList<>();
		intparams.add(0,getCx());
		intparams.add(1,getCy());
		intparams.add(2,getRx());
		intparams.add(3,getRy());
		CreateEclipse eclipse=new CreateEclipse();
		return eclipse.create(intparams,getFill(),getID(params.get(TERMS.CANVASID_INDEX)));
	}

	public ICreateShape getShapeCreation() {
		return shapeCreation;
	}

	public void setShapeCreation(ICreateShape shapeCreation) {
		this.shapeCreation = shapeCreation;
	}

	public int getCx() {
		return cx;
	}

	public void setCx(int cx) {
		this.cx = cx;
	}

	public int getCy() {
		return cy;
	}

	public void setCy(int cy) {
		this.cy = cy;
	}


	public Colours getFill() {
		return fill;
	}

	public void setFill(String fill) {
		this.fill=Colours.BLUE;
		for(Colours x:Colours.values()){
			if(x.toString().equals(fill.toUpperCase())) this.fill=x;
		}
	}
	private String getID(String canvasId){
		Random r=new Random();
		return  canvasId+"_"+r.nextInt(999)+1;
	}

	public Logger getLog() {
		return log;
	}

	public void setLog(Logger log) {
		this.log = log;
	}

	public int getRx() {
		return rx;
	}

	public void setRx(int rx) {
		this.rx = rx;
	}

	public int getRy() {
		return ry;
	}

	public void setRy(int ry) {
		this.ry = ry;
	}

	public int getStroke_width() {
		return stroke_width;
	}

	public void setStroke_width(int stroke_width) {
		this.stroke_width = stroke_width;
	}

	public void setFill(Colours fill) {
		this.fill = fill;
	}

	public Colours getStroke() {
		return stroke;
	}

	public void setStroke(Colours stroke) {
		this.stroke = stroke;
	}

	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}
}