package serialization;

import exceptions.serverexceptions.SerializationException;
import layers.LayerRepo;
import library.TERMS;
import models.Layer;
import models.shapes.IShapes;

import java.io.IOException;
import java.util.logging.Logger;

public class ReadClientData {
	private String path;
	private Logger log= Logger.getLogger(ReadClientData.class.getName());
	AccessClientData data;

	/**
	 * Reads all available Clientdata with functions
	 * @param clientID String clientid
	 */
	public ReadClientData(String clientID) {
		this.path = TERMS.DATAPATH+clientID;
		data=new AccessClientData(path);
	}

	/**
	 * gives a Layerobject
	 * @param layerId string layerid
	 * @return Layer
	 * @throws SerializationException error while handling Clientdata
	 */
	public Layer readLayer(String layerId) throws SerializationException {
		Layer layer=null;
		try {
			layer=data.readLayer(layerId);
		} catch (ClassNotFoundException | IOException e) {
			log.warning(e.toString());
			throw new SerializationException(e.toString());
		}
		return layer;
	}

	/**
	 * returns the Shape
	 * @param id objectid
	 * @return IShape, auslesen mit ISHAPER instance of X
	 * @throws SerializationException error while handling Clientdata
	 */
	public IShapes getShapeById(String id) throws SerializationException {
		IShapes shape=null;
		try {
			shape=data.getShape(id);
		} catch (ClassNotFoundException | IOException e) {
			log.warning(e.toString());
			throw new SerializationException(e.toString());
		}

		return null;
	}

	/**
	 *
	 * @return LayerRepo object
	 * @throws SerializationException error while handling Clientdata
	 */
	public LayerRepo getLayerRepoOfClient() throws SerializationException {
		try {
			return data.getLayerRepo();
		} catch (ClassNotFoundException | IOException e) {
			log.warning(e.toString());
			throw new SerializationException(e.toString());
		}
	}
}
